# -*- coding: cp1252 -*-
"""Information sur une map multijoueur"""

import distob
import gameinfo
import network
import EugUtil
import Accesseur
import time
import effet
import itertools

def get_map_path_from_map_id (map_id):
    return Accesseur.EvalueAccesseur('TAccesseurGetMapPath', MapID = map_id)

class MapInfo(object):
    def __init__(self, binary_info):
        self.map_name = binary_info.MapName
        self.map_name_dev = binary_info.MapNameDev
        self.map_description = binary_info.MapDescription
        self.texture_name = binary_info.TextureName
        self.nombre_joueurs = binary_info.MaxPlayers
        self.is_packed_map = binary_info.IsPackedMap
        self.mapid = binary_info.MapID
        self.file_size = binary_info.FileSize
        self.pack_path = binary_info.Packpath
        self.startingpoint_list = [(i.PositionX,i.PositionY,i.FavoredTeam2,i.FavoredTeam3,i.FavoredTeam4) for i in binary_info.StartingPointList]
        self.icone_list = [(i.IconeIndex, i.PositionX, i.PositionY) for i in binary_info.PreviewMaskIconeList]
        self.tags_pour_la_carte = binary_info.TagsPourLaCarte
        self.world_size = binary_info.WorldSizeX, binary_info.WorldSizeY
    def set_binary_info(self, binary_info):
        binary_info.MapName = self.map_name
        binary_info.MapNameDev = self.map_name_dev
        binary_info.MapDescription = self.map_description
        binary_info.TextureName = self.texture_name
        binary_info.MaxPlayers = self.nombre_joueurs
        binary_info.IsPackedMap = self.is_packed_map
        binary_info.MapID = self.mapid
        binary_info.FileSize = self.file_size
        binary_info.Packpath = self.pack_path
        binary_info.TagsPourLaCarte = self.tags_pour_la_carte
        binary_info.WorldSizeX, binary_info.WorldSizeY = self.world_size
        for i in binary_info.StartingPointList:
            i.EugFree()
        binary_info.StartingPointList.clear()
        for i in binary_info.PreviewMaskIconeList:
            i.EugFree()
        binary_info.PreviewMaskIconeList.clear()
        for (px, py, ft2, ft3, ft4) in self.startingpoint_list:
            binary_info.StartingPointList.append(EugUtil.MakeObject('TStartingPointPosition',
                                                                        PositionX = px,
                                                                        PositionY = py,
                                                                        FavoredTeam2 = ft2,
                                                                        FavoredTeam3 = ft3,
                                                                        FavoredTeam4 = ft4
                                                                    )
            )
        for (iindex, px, py) in self.icone_list:
            binary_info.PreviewMaskIconeList.append(EugUtil.MakeObject('TPreviewMaskIconeElement',
                                                                           IconeIndex = iindex,
                                                                           PositionX = px,
                                                                           PositionY = py
                                                                      )
            )

map_manager = None
download_helper = None

class MapManager(distob.distob):
    current_map = None    
    def init(self):
        global map_manager
        map_manager = self
    def _on_current_map(self, old_val, new_val):
        global download_helper
        if download_helper is not None:
            download_helper.EugFree()
            download_helper = None
        gi = gameinfo.GameInfo.info
        if self.current_map.is_packed_map:
            download_helper = EugUtil.MakeObject( 'TDownloadHelper',
                                                  FileSize     = self.current_map.file_size,
                                                  MD5CheckSUM  = self.current_map.mapid[:32],
                                                  FullFileName = get_map_path_from_map_id(self.current_map.mapid) )
        gi.interface_data.map_manager.LastSelectedMapMapID = new_val.mapid
        gi.set_max_players(new_val.nombre_joueurs)
        gi.interface_data.map_id_object.Value = self.current_map.mapid
        network.state_changed()

def is_download_complete():
    return download_helper is None or download_helper.Finished

last_download_time = 0
was_download_finished = True
def check_download():
    global last_download_time, was_download_finished
    if last_download_time < time.time() - 1:
        last_download_time = time.time()
        if is_download_complete():
            advance = 1.0
            if not was_download_finished:
                effet.RunEffetInstantane(gameinfo.GameInfo.info.interface_data.evenement_interface)
            was_download_finished = True
        else:
            advance = download_helper.Pourcentage
            was_download_finished = False
        if gameinfo.get_self_player() is not None and gameinfo.get_self_player().download_state != advance:
            gameinfo.get_self_player().update_download(advance)

def get_minimap_path():
    if map_manager and map_manager.current_map and is_download_complete():
        if map_manager.current_map.is_packed_map:
            assert download_helper
            return u'%' + download_helper.FileName + u'%\\CustomMap\\Preview.jpeg'
        else:
            return map_manager.current_map.mapid + u'\\Preview.jpeg'
    else:
        return u'Data\\RMTexture2\\ToutTransparent.TGA'

def get_minimap_mask():
    if map_manager and map_manager.current_map and is_download_complete():
        if map_manager.current_map.is_packed_map:
            assert download_helper
            return u'%' + download_helper.FileName + u'%\\CustomMap\\Preview_Mask.TGA'
        else:
            return map_manager.current_map.mapid + u'\\Preview_Mask.TGA'
    else:
        return u'Data\\RMTexture2\\ToutTransparent.TGA'

def get_max_player_count():
    try:
        if gameinfo.GameInfo.info:
            return gameinfo.GameInfo.info.nombre_slots
    except ReferenceError, e:
        pass
    return 8

def get_nombre_slots_libres():
    try:
        if gameinfo.GameInfo.info:
            return gameinfo.GameInfo.info.get_nombre_slots_libres()
    except ReferenceError, e:
        pass
    return 8

def get_nombre_joueurs_map():
    try:
        if gameinfo.GameInfo.info:
            return gameinfo.GameInfo.info.nombre_joueurs
    except ReferenceError, e:
        pass
    return 8

def get_map_id():
    if map_manager and map_manager.current_map:
        return unicode(map_manager.current_map.mapid)
    else:
        return u''

def get_map_name():
    if map_manager and map_manager.current_map:
        if map_manager.current_map.is_packed_map:
            return map_manager.current_map.map_name_dev
        else:
            return map_manager.current_map.map_name
    else:
        return u''

def filter_maps( official_maps_all, custom_maps_all,
                 official_maps_current, custom_maps_current,
                 current_mode ):
    official_maps_current.clear()
    custom_maps_current.clear()
    game_mode = ('deathmatch', 'marineoneisdown', 'scudlauncher', 'navy', 'deathmatch')
    for map in official_maps_all:
        if game_mode[current_mode.Published_KillableEugObject.Index] in map.EugObject.TagsPourLaCarte.split(" "):
            official_maps_current.append(map)
    for map in custom_maps_all:
        if game_mode[current_mode.Published_KillableEugObject.Index] in map.EugObject.TagsPourLaCarte.split(" "):
            custom_maps_current.append(map)

def filter_modes( official_maps_all, custom_maps_all,
                  modes_all, modes_current ):
    modes_current.clear()
    game_mode = ('deathmatch', 'marineoneisdown', 'scudlauncher', 'navy', 'deathmatch')
    for mode in modes_all:
        for map in itertools.chain(official_maps_all, custom_maps_all):
            if game_mode[mode.Index] in map.EugObject.TagsPourLaCarte.split(" "):
                modes_current.append(mode)
                break
