# -*- coding: cp1252 -*-

import ndf
import math
from multiplayer.profile_rank import get_profile_rank

date_debut_partie = ndf.DBGetA('DefaultConst/ObjectString_DateDebutPartie')
date_fin_partie   = ndf.DBGetA('DefaultConst/ObjectString_DateFinPartie')
player_win        = ndf.DBGetA('DefaultConst/ObjectStringPlayerWin')
player_lost       = ndf.DBGetA('DefaultConst/ObjectStringPlayerLost')
LOSS_MULTIPLIER   = [0.1, 0.11, 0.25, 0.43, 0.67]

def expected_ladder_level(win, loss):
    return min(math.sqrt((win + loss) * (win + 1) / (loss + 1)), 6)

def probability_of_win(ell_self, ell_opponent, coeff = 22.5):
    return 1.0 / (1 + math.pow(10.0, (ell_self - ell_opponent) / coeff))

def probability_of_loss(ell_self, ell_opponent):
    if ell_self >= ell_opponent:
        return probability_of_win(ell_self, ell_opponent)
    else:
        return probability_of_win(ell_self, ell_opponent, 7.6)

def update_singleplayer(profile, unit_balise_camp):

    self_camp = unit_balise_camp.camp
    if self_camp.NiveauIA == 0 or self_camp.TeamNumber == -1:
        return

    fin = unit_balise_camp[date_fin_partie]
    debut = unit_balise_camp[date_debut_partie]
    profile.TotalDutyTimeInSec = profile.TotalDutyTimeInSec + int(fin - debut)

def filter_camp_and_profile_list(camp_list, profiles_list):
    return [ (c, p) for c, p in zip(camp_list, profiles_list) if c.NiveauIA and c.TeamNumber != -1 ]

class Team:

    def __init__(self):
        self.win = 0
        self.loss = 0
        self.players = []

    def add_player(self, player):
        self.win += player.Victoire
        self.loss += player.Defaite
        self.players.append(player)

def update_multiplayer(profile, unit_balise_camp, profiles_list, camp_list):

    self_camp = unit_balise_camp.camp
    if self_camp.NiveauIA == 0 or self_camp.TeamNumber == -1:
        return

    teams = {}
    for c, p in filter_camp_and_profile_list(camp_list, profiles_list):
        if p.TeamNumber != c.TeamNumber :
            return
        teams.setdefault(p.TeamNumber, Team()).add_player(p)
        if profile.FullName == p.FullName:
            profile.TeamNumber = p.TeamNumber

    total_ell = 0
    for team in teams.itervalues():
        team.ell = expected_ladder_level(team.win, team.loss)
        total_ell += team.ell
        print 'Team%s.ell = %s' % (team.players[0].TeamNumber, team.ell)

    if len(teams) >= 2:
        team = teams[profile.TeamNumber]
        ell_opponents = (total_ell - team.ell) / (len(teams) - 1.)
        if unit_balise_camp[player_win]:
            probability = probability_of_win(team.ell, ell_opponents)
            profile.Grade1vs1 = profile.Grade1vs1 + int(math.floor(200 * probability))
            print 'Player: %s gained %s experience' % (profile.FullName, 200 * probability)
        if unit_balise_camp[player_lost]:
            probability = probability_of_loss(team.ell, ell_opponents)
            rank = get_profile_rank(profile)
            if 0 <= rank < len(LOSS_MULTIPLIER):
                probability = probability * LOSS_MULTIPLIER[rank]
            profile.Grade1vs1 = max(profile.Grade1vs1 - int(math.floor(200 * probability)), 0)
            print 'Player: %s lost %s experience' % (profile.FullName, 200 * probability)

    profile.Victoire = profile.Victoire + unit_balise_camp[player_win]
    profile.Defaite = profile.Defaite + unit_balise_camp[player_lost]
    update_singleplayer(profile, unit_balise_camp)

